--======================================================--
--                                                      --
--  NORTHEASTERN UNIVERSITY                             --
--  DEPARTMENT OF ELECTRICAL AND COMPUTER ENGINEERING   --
--  RAPID PROTOTYPING LABORATORY                        --
--                                                      --
--  FILE         | add_sub.vhd                          --
--  -------------+------------------------------------  --
--  DESCRIPTION  | Module to perform addition or        --
--               | subtraction.                         --
--  -------------+------------------------------------  --
--  AUTHOR       | Pavle Belanovic                      --
--  -------------+------------------------------------  --
--  DATE         | 20 June 2002                         --
--                                                      --
--======================================================--

--******************************************************************************--
--                                                                              --
--	Copyright (C) 2000 Pavle Belanovic                                          --
--                                                                              --
--	This program is free software; you can redistribute it and/or               --
--	modify it under the terms of the GNU General Public License                 --
--	as published by the Free Software Foundation; either version 2              --
--	of the License, or (at your option) any later version.                      --
--                                                                              --
--	This program is distributed in the hope that it will be useful,             --
--	but WITHOUT ANY WARRANTY; without even the implied warranty of              --
--	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the               --
--	GNU General Public License for more details.                                --
--                                                                              --
--	You should have received a copy of the GNU General Public License           --
--	along with this program; if not, write to the Free Software                 --
--	Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA. --
--                                                                              --
--******************************************************************************--

--======================================================--
--                      LIBRARIES                       --
--======================================================--

-- IEEE Libraries --
library IEEE;
use IEEE.std_logic_1164.all;
use IEEE.std_logic_arith.all;
use IEEE.std_logic_unsigned.all;

-- float
library work;
use work.float_pkg.all;

----------------------------------------------------------
--                  Add/Sub Module                      --
----------------------------------------------------------
entity add_sub is
	generic
	(
		man_bits			:	integer	:=	0
	);
	port
	(
		--inputs
		F1						:	in		std_logic_vector(man_bits downto 0);
		F2						:	in		std_logic_vector(man_bits downto 0);
		READY					:	in		std_logic;
		OP						:	in		std_logic;
		CLK						:	in		std_logic;
		--outputs
		F_OUT					:	out		std_logic_vector(man_bits downto 0)	:=	(others=>'0');
		OVERFLOW			:	out		std_logic														:=	'0';
		DONE					:	out		std_logic														:=	'0'
	);
end add_sub;

----------------------------------------------------------
--                  Add/Sub Module                      --
----------------------------------------------------------
architecture add_sub_arch of add_sub is
	--SIGNALS
	signal	nop			:	std_logic														:=	'0';
	signal	cout		:	std_logic														:=	'0';
	signal	nf2			:	std_logic_vector(man_bits downto 0)	:=	(others=>'0');
	signal	op2			:	std_logic_vector(man_bits downto 0)	:=	(others=>'0');
	signal	ov			:	std_logic														:=	'0';
	signal	result	:	std_logic_vector(man_bits downto 0)	:=	(others=>'0');
begin
	--ASYNCHRONOUS
	--signal connections
	nop		<=	NOT(OP);
	nf2		<=	NOT(F2);
	ov		<=	cout AND OP;
	--component instantiations
	pm	:	parameterized_mux
		generic map
		(
			bits	=>	man_bits+1
		)
		port map
		(
			A			=>	F2,
			B			=>	nf2,
			S			=>	OP,
			O			=>	op2
		);
	pa	:	parameterized_adder
		generic map
		(
			bits	=>	man_bits+1
		)
		port map
		(
			A			=>	F1,
			B			=>	op2,
			CIN		=>	nop,
			S			=>	result,
			COUT	=>	cout
		);
	--SYNCHRONOUS
	main: process (CLK)
	begin
		if(rising_edge(CLK)) then
			DONE			<=	READY;
			OVERFLOW	<=	ov;
			F_OUT			<=	result;
		end if;--CLK
	end process;--main
	
end add_sub_arch; --end of architecture

