--======================================================--
--                                                      --
--  NORTHEASTERN UNIVERSITY                             --
--  DEPARTMENT OF ELECTRICAL AND COMPUTER ENGINEERING   --
--  RAPID PROTOTYPING LABORATORY                        --
--                                                      --
--  FILE         | one_bit_comparator.vhd               --
--  -------------+------------------------------------  --
--  DESCRIPTION  | 1-bit comparator                     --
--  -------------+------------------------------------  --
--  AUTHOR       | Pavle Belanovic                      --
--  -------------+------------------------------------  --
--  DATE         | 20 June 2002                         --
--                                                      --
--======================================================--

--******************************************************************************--
--                                                                              --
--	Copyright (C) 2000 Pavle Belanovic                                          --
--                                                                              --
--	This program is free software; you can redistribute it and/or               --
--	modify it under the terms of the GNU General Public License                 --
--	as published by the Free Software Foundation; either version 2              --
--	of the License, or (at your option) any later version.                      --
--                                                                              --
--	This program is distributed in the hope that it will be useful,             --
--	but WITHOUT ANY WARRANTY; without even the implied warranty of              --
--	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the               --
--	GNU General Public License for more details.                                --
--                                                                              --
--	You should have received a copy of the GNU General Public License           --
--	along with this program; if not, write to the Free Software                 --
--	Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA. --
--                                                                              --
--******************************************************************************--

--======================================================--
--                      LIBRARIES                       --
--======================================================--

-- IEEE Libraries --
library IEEE;
use IEEE.std_logic_1164.all;
use IEEE.std_logic_arith.all;
use IEEE.std_logic_unsigned.all;

-- float
--library PEX_Lib;
--use PEX_Lib.float_pkg.all;

----------------------------------------------------------
--                One bit comparator                    --
----------------------------------------------------------
entity one_bit_comparator is
	port
	(
		--inputs
		A						:	in		std_logic;
		B						:	in		std_logic;
		GT					:	in		std_logic;
		EQ					:	in		std_logic;
		LT					:	in		std_logic;
		--outputs
		A_GT_B			:	out		std_logic		:=	'0';
		A_EQ_B			:	out		std_logic		:=	'0';
		A_LT_B			:	out		std_logic		:=	'0'
	);
end one_bit_comparator;

----------------------------------------------------------
--                One bit comparator                    --
----------------------------------------------------------
architecture one_bit_comparator_arch of one_bit_comparator is
	signal	im1		:		std_logic	:=	'0';
	signal	im2		:		std_logic	:=	'0';
	signal	im3		:		std_logic	:=	'0';
	signal	im4		:		std_logic	:=	'0';
	signal	im5		:		std_logic	:=	'0';
	signal	im6		:		std_logic	:=	'0';
	signal	im7		:		std_logic	:=	'0';
	signal	im8		:		std_logic	:=	'0';
	signal	na		:		std_logic	:=	'0';
	signal	nb		:		std_logic	:=	'0';
begin
	na	<=	NOT(A);
	nb	<=	NOT(B);
	im1	<=	A NAND GT;
	im2	<=	nb NAND GT;
	im3	<=	A NAND nb;
	im4	<=	NOT(A AND (B AND EQ));
	im5	<=	NOT(na AND (nb AND EQ));
	im6	<=	na NAND B;
	im7	<=	na NAND LT;
	im8	<=	B NAND LT;
	--outputs
	A_GT_B	<=	NOT(im1 AND (im2 AND im3));
	A_EQ_B	<=	im4 NAND im5;
	A_LT_B	<=	NOT(im6 AND (im7 AND im8));
end one_bit_comparator_arch; -- end of architecture


