--======================================================--
--                                                      --
--  NORTHEASTERN UNIVERSITY                             --
--  DEPARTMENT OF ELECTRICAL AND COMPUTER ENGINEERING   --
--  RAPID PROTOTYPING LABORATORY                        --
--                                                      --
--  FILE         | round_add.vhd                        --
--  -------------+------------------------------------  --
--  DESCRIPTION  | Module that performs rounding        --
--               | addition.                            --
--  -------------+------------------------------------  --
--  AUTHOR       | Pavle Belanovic                      --
--  -------------+------------------------------------  --
--  DATE         | 20 June 2002                         --
--                                                      --
--======================================================--

--******************************************************************************--
--                                                                              --
--	Copyright (C) 2000 Pavle Belanovic                                          --
--                                                                              --
--	This program is free software; you can redistribute it and/or               --
--	modify it under the terms of the GNU General Public License                 --
--	as published by the Free Software Foundation; either version 2              --
--	of the License, or (at your option) any later version.                      --
--                                                                              --
--	This program is distributed in the hope that it will be useful,             --
--	but WITHOUT ANY WARRANTY; without even the implied warranty of              --
--	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the               --
--	GNU General Public License for more details.                                --
--                                                                              --
--	You should have received a copy of the GNU General Public License           --
--	along with this program; if not, write to the Free Software                 --
--	Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA. --
--                                                                              --
--******************************************************************************--

--======================================================--
--                      LIBRARIES                       --
--======================================================--

-- IEEE Libraries --
library IEEE;
use IEEE.std_logic_1164.all;
use IEEE.std_logic_arith.all;
use IEEE.std_logic_unsigned.all;

-- float
library work;
use work.float_pkg.all;

----------------------------------------------------------
--                 Rounding addition                    --
----------------------------------------------------------
entity round_add is
	generic
	(
		man_bits_in		:	integer	:=	0;
		man_bits_out	:	integer	:=	0
	);
	port
	(
		--inputs
		MAN_IN				:	in		std_logic_vector(man_bits_in-1 downto 0);
		READY					:	in		std_logic;
		CLK						:	in		std_logic;
		ENABLE				:	in		std_logic;
		EXCEPTION_IN	:	in		std_logic;
		--outputs
		MAN_OUT				:	out		std_logic_vector(man_bits_out-1 downto 0)	:=	(others=>'0');
		EXCEPTION_OUT	:	out		std_logic																	:=	'0';
		DONE					:	out		std_logic																	:=	'0'
	);
end round_add;

----------------------------------------------------------
--                 Rounding addition                    --
----------------------------------------------------------
architecture round_add_arch of round_add is
	--SIGNALS
	signal	slice		:	std_logic_vector(man_bits_out downto 0)		:=	(others=>'0');
	signal	man_int	:	std_logic_vector(man_bits_out downto 0)		:=	(others=>'0');
	signal	exc_int	:	std_logic																	:=	'0';
	signal	exc_pa	:	std_logic																	:=	'0';
	signal	zeros		:	std_logic_vector(man_bits_out downto 0)		:=	(others=>'0');
begin
	--ASYNCHRONOUS
	--component instantiation
	pa	: parameterized_adder
		generic map
		(
			bits	=>	man_bits_out+1
		)
		port map
		(
			A			=>	slice,
			B			=>	zeros,
			CIN		=>	ENABLE,
			S			=>	man_int,
			COUT	=>	exc_pa
		);
	--signal connections
	slice		<=	MAN_IN(man_bits_in-2 downto man_bits_in-man_bits_out-2);
	exc_int	<=	exc_pa OR EXCEPTION_IN;
	--SYNCHRONOUS
	main: process (CLK)
	begin
		if(rising_edge(CLK)) then
			MAN_OUT				<=	man_int(man_bits_out downto 1);
			EXCEPTION_OUT	<=	exc_int;
			DONE					<=	READY;
		end if; --CLK
	end process; --main
end round_add_arch; -- end of architecture
