--======================================================--
--                                                      --
--  NORTHEASTERN UNIVERSITY                             --
--  DEPARTMENT OF ELECTRICAL AND COMPUTER ENGINEERING   --
--  RAPID PROTOTYPING LABORATORY                        --
--                                                      --
--  FILE         | shifter_control.vhd                  --
--  -------------+------------------------------------  --
--  DESCRIPTION  | Module for controlling variable      --
--               | shifter.                             --
--  -------------+------------------------------------  --
--  AUTHOR       | Pavle Belanovic                      --
--  -------------+------------------------------------  --
--  DATE         | 20 June 2002                         --
--                                                      --
--======================================================--

--******************************************************************************--
--                                                                              --
--	Copyright (C) 2000 Pavle Belanovic                                          --
--                                                                              --
--	This program is free software; you can redistribute it and/or               --
--	modify it under the terms of the GNU General Public License                 --
--	as published by the Free Software Foundation; either version 2              --
--	of the License, or (at your option) any later version.                      --
--                                                                              --
--	This program is distributed in the hope that it will be useful,             --
--	but WITHOUT ANY WARRANTY; without even the implied warranty of              --
--	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the               --
--	GNU General Public License for more details.                                --
--                                                                              --
--	You should have received a copy of the GNU General Public License           --
--	along with this program; if not, write to the Free Software                 --
--	Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA. --
--                                                                              --
--******************************************************************************--

--======================================================--
--                      LIBRARIES                       --
--======================================================--

-- IEEE Libraries --
library IEEE;
use IEEE.std_logic_1164.all;
use IEEE.std_logic_arith.all;
use IEEE.std_logic_unsigned.all;

-- float
library work;
use work.float_pkg.all;

----------------------------------------------------------
--              		  Shifter Control	                  --
----------------------------------------------------------
entity shifter_control is
	generic
	(
		exp_bits			:	integer	:=	0;
		select_bits		:	integer	:=	0
	);
	port
	(
		--inputs
		DELTA					:	in		std_logic_vector(exp_bits-1 downto 0);
		--outputs
		CLEAR					:	out		std_logic;
		SHIFT					:	out		std_logic_vector(select_bits-1 downto 0)
	);
end shifter_control;

----------------------------------------------------------
--              		  Shifter Control	                  --
----------------------------------------------------------
architecture shifter_control_arch of shifter_control is
begin
	equal	:	if(exp_bits=select_bits) generate --limiting case
		CLEAR	<=	'0';
		SHIFT	<=	DELTA;
	end generate;--equal
	large	:	if(exp_bits>select_bits) generate --common case
		--if more than one extra bit
		more_than_one	:	if(exp_bits>select_bits+1) generate
			or_gate	:	parameterized_or_gate
				generic map
				(
					bits	=>	exp_bits-select_bits
				)
				port map
				(
					A			=>	DELTA(exp_bits-1 downto select_bits),
					O			=>	CLEAR
				);
		end generate;--more than one bit
		--if only one extra bit
		only_one	:	if(exp_bits=select_bits+1) generate
			CLEAR	<=	DELTA(exp_bits-1);
		end generate;--only one bit
		SHIFT	<=	DELTA(select_bits-1 downto 0);
	end generate;--large
	small	:	if(exp_bits<select_bits) generate --extreme case
		CLEAR																	<=	'0';
		SHIFT(exp_bits-1 downto 0)						<=	DELTA;
		SHIFT(select_bits-1 downto exp_bits)	<=	(others=>'0');
	end generate;--small
end shifter_control_arch; -- end of architecture

