--======================================================--
--                                                      --
--  NORTHEASTERN UNIVERSITY                             --
--  DEPARTMENT OF ELECTRICAL AND COMPUTER ENGINEERING   --
--  RAPID PROTOTYPING LABORATORY                        --
--                                                      --
--  FILE         | swap.vhd                             --
--  -------------+------------------------------------  --
--  DESCRIPTION  | Module for sorting two inputs into   --
--               | larger and smaller.                  --
--  -------------+------------------------------------  --
--  AUTHOR       | Pavle Belanovic                      --
--  -------------+------------------------------------  --
--  DATE         | 20 June 2002                         --
--                                                      --
--======================================================--

--******************************************************************************--
--                                                                              --
--	Copyright (C) 2000 Pavle Belanovic                                          --
--                                                                              --
--	This program is free software; you can redistribute it and/or               --
--	modify it under the terms of the GNU General Public License                 --
--	as published by the Free Software Foundation; either version 2              --
--	of the License, or (at your option) any later version.                      --
--                                                                              --
--	This program is distributed in the hope that it will be useful,             --
--	but WITHOUT ANY WARRANTY; without even the implied warranty of              --
--	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the               --
--	GNU General Public License for more details.                                --
--                                                                              --
--	You should have received a copy of the GNU General Public License           --
--	along with this program; if not, write to the Free Software                 --
--	Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA. --
--                                                                              --
--******************************************************************************--

--======================================================--
--                      LIBRARIES                       --
--======================================================--

-- IEEE Libraries --
library IEEE;
use IEEE.std_logic_1164.all;
use IEEE.std_logic_arith.all;
use IEEE.std_logic_unsigned.all;

-- float
library work;
use work.float_pkg.all;

----------------------------------------------------------
--                     Swap module                      --
----------------------------------------------------------
entity swap is
	generic
	(
		exp_bits		:	integer	:=	0;
		man_bits		:	integer	:=	0
	);
	port
	(
		--inputs
		IN1						:	in		std_logic_vector((exp_bits+man_bits) downto 0);
		IN2						:	in		std_logic_vector((exp_bits+man_bits) downto 0);
		READY					:	in		std_logic;
		CLK						:	in		std_logic;
		--outputs
		OUT1					:	out		std_logic_vector((exp_bits+man_bits) downto 0)	:=	(others=>'0');
		OUT2					:	out		std_logic_vector((exp_bits+man_bits) downto 0)	:=	(others=>'0');
		DONE					:	out		std_logic																				:=	'0'
	);
end swap;

----------------------------------------------------------
--                     Swap module                      --
----------------------------------------------------------
architecture swap_arch of swap is
	--SIGNALS
	signal	a_gt_b_e	:	std_logic		:=	'0';
	signal	a_eq_b_e	:	std_logic		:=	'0';
	signal	a_lt_b_e	:	std_logic		:=	'0';
	signal	a_gt_b_f	:	std_logic		:=	'0';
	signal	a_eq_b_f	:	std_logic		:=	'0';
	signal	a_lt_b_f	:	std_logic		:=	'0';
	signal	mux_ctrl	:	std_logic		:=	'0';
	signal	nmux_ctrl	:	std_logic		:=	'0';
	signal	man1			:	std_logic_vector(man_bits-1 downto 0)	:=	(others=>'0');
	signal	man2			:	std_logic_vector(man_bits-1 downto 0)	:=	(others=>'0');
	signal	exp1			:	std_logic_vector(exp_bits-1 downto 0)	:=	(others=>'0');
	signal	exp2			:	std_logic_vector(exp_bits-1 downto 0)	:=	(others=>'0');
	signal	large			:	std_logic_vector((exp_bits+man_bits) downto 0)	:=	(others=>'0');
	signal	small			:	std_logic_vector((exp_bits+man_bits) downto 0)	:=	(others=>'0');
begin
	--concurrent
	--connect signals
	exp1			<=	IN1(exp_bits+man_bits-1 downto man_bits);
	exp2			<=	IN2(exp_bits+man_bits-1 downto man_bits);
	man1			<=	IN1(man_bits-1 downto 0);
	man2			<=	IN2(man_bits-1 downto 0);
	nmux_ctrl	<=	NOT(mux_ctrl);
	--instantiate the components
	exp_comparator	:	parameterized_comparator
		generic map
		(
			bits				=>	exp_bits
		)
		port map
		(
			--inputs
			A						=>	exp1,
			B						=>	exp2,
			--outputs
			A_GT_B			=>	a_gt_b_e,
			A_EQ_B			=>	a_eq_b_e,
			A_LT_B			=>	a_lt_b_e
		);
	man_comparator	:	parameterized_comparator
		generic map
		(
			bits				=>	man_bits
		)
		port map
		(
			--inputs
			A						=>	man1,
			B						=>	man2,
			--outputs
			A_GT_B			=>	a_gt_b_f,
			A_EQ_B			=>	a_eq_b_f,
			A_LT_B			=>	a_lt_b_f
		);
	control	:	mux2
		port map
		(
			--inputs
			A						=>	a_lt_b_f,
			B						=>	a_lt_b_e,
			S						=>	a_eq_b_e,
			--outputs
			O						=>	mux_ctrl
		);
	out_mux1	:	parameterized_mux
		generic map
		(
			bits				=>	exp_bits+man_bits+1
		)
		port map
		(
			--inputs
			A						=>	IN1,
			B						=>	IN2,
			S						=>	nmux_ctrl,
			--outputs
			O						=>	large
		);
	out_mux2	:	parameterized_mux
		generic map
		(
			bits				=>	exp_bits+man_bits+1
		)
		port map
		(
			--inputs
			A						=>	IN1,
			B						=>	IN2,
			S						=>	mux_ctrl,
			--outputs
			O						=>	small
		);
	
	--sequential
	main	:	process(CLK)
	begin
		if(rising_edge(CLK)) then
			DONE	<=	READY;
			OUT1	<=	large;
			OUT2	<=	small;
		end if;--CLK
	end process; --main
end swap_arch; -- end of architecture

